from datetime import datetime, timedelta
import logging
import re
import os

from django.db import models
from django.utils import timezone
from django.utils.dateparse import parse_datetime
from django.core.exceptions import ValidationError
from django.conf import settings

from . import utils
from .api import FasAd

from .querysets import RealEstateQuerySet


class BaseManager(models.Manager):

    def __init__(self):
        super(BaseManager, self).__init__()

        self.fasad = FasAd(
            company_id=settings.FASAD_COMPANY_ID,
            office_id=settings.FASAD_OFFICE_ID)

    @staticmethod
    def _none_or_string_to_int(value):
        if value is None:
            return value
        else:
            return int(value)


class FasAdRealtorManager(BaseManager):

    def synchronize(self, loggging=False):
        """
        Fetch and update or create all realtors from FasAd.
        """
        logger = logging.getLogger(__name__)
        logger.warning('Synchronizing realtors.')

        realtors_json = self.fasad.realtors()

        logger.warning('Found {} realtors.'.format(len(realtors_json)))

        for index, realtor in enumerate(realtors_json):
            fasad_id = realtor['@id']
            logger.warning('Updating or creating realtor #{}.'.format(fasad_id))
            self._update_or_create_from_json(realtor, index)

        fasad_ids = [realtor['@id'] for realtor in realtors_json]
        self.exclude(fasad_id__in=fasad_ids).delete()

        logger.warning('Done synchronizing realtors.')

    def _update_or_create_from_json(self, json_item, priority):
        return self.update_or_create(
            fasad_id=json_item['@id'],
            defaults={
                'fasad_id': int(json_item['@id']),
                'priority': priority,
                'first_name': json_item['@firstname'],
                'last_name': json_item['@lastname'],
                'title': json_item['@title'],
                'phone': json_item['@phoneString'],
                'email': json_item['@email'],
                'image_url': json_item['@image'],
                'public': self._none_or_string_to_int(json_item['@public']),
            }
        )


class FasAdRealEstateBaseManager(BaseManager):

    def synchronize(self, only_modified=True):
        """
        Fetch real estates from FasAd and create/update new and modified ones.
        """
        logger = logging.getLogger(__name__)

        if only_modified:
            logger.warning('Synchronizing only modified real estates.')
        else:
            logger.warning('Synchronizing all real estates.')

        real_estates_json = self.fasad.real_estates()
        fasad_ids = [real_estate['@id'] for real_estate in real_estates_json]
        logger.warning('Found {} real estates.'.format(len(real_estates_json)))

        if only_modified:
            real_estates_json = self._filter_modified(real_estates_json)

        for real_estate_json in real_estates_json:
            self.update_or_create_from_fasad(fasad_id=real_estate_json['@id'], published=True)

        removed_real_estates = self.exclude(fasad_id__in=fasad_ids)
        logger.warning('Found {} removed real estates.'.format(len(removed_real_estates)))
        removed_real_estates.update(published=False)

    def _filter_modified(self, real_estates_json):
        """
        Get a list of FasAd IDs for the real estates that have changed since the last iteration.
        """
        modified_items = [
            item for item in real_estates_json if self._is_modified_since(item)
        ]

        return modified_items

    def _is_modified_since(self, json_dict, since_minutes=86400):
        last_modified_datetime = self._get_last_modified_datetime(json_dict)
        since_datetime = datetime.now()-timedelta(minutes=since_minutes)

        return last_modified_datetime > since_datetime

    def _get_last_modified_datetime(self, json_dict):
        datetime_format = '%Y-%m-%d %H:%M:%S'

        try:
            last_modified_datetime = datetime.strptime(json_dict['@lastPublished'],
                                                       datetime_format)
        except ValueError:
            last_modified_datetime = datetime.strptime(json_dict['@firstPublished'],
                                                       datetime_format)

        return last_modified_datetime

    def update_or_create_from_fasad(self, fasad_id, published=None):
        logger = logging.getLogger(__name__)
        logger.warning('Updating or creating real estate #{}.'.format(fasad_id))
        from .models import Realtor

        real_estate_json = self.fasad.real_estate(fasad_id)

        return self._update_or_create_from_json(real_estate_json, published)

    def _update_or_create_from_json(self, json_item, published):
        """
        May throw Realtor.DoesNotExist.
        """
        from .forms import RealEstateForm
        from .models import Realtor, Association, Showing, Bid, Image, Document, Media

        d = json_item
        values = dict()
        fasad_id = int(d['@id'])

        try:
            values['realtor'] = self._get_realtor_pk(d)
        except Realtor.DoesNotExist:
            pass

        values['fasad_id'] = fasad_id
        values['status'] = d['@status']
        values['address'] = d['Location']['@address']
        values['district'] = d['Location']['@district']
        values['city'] = d['Location']['@city']
        values['country'] = d['Location']['@country']
        values['longitude'] = d['Location']['@lon']
        values['latitude'] = d['Location']['@lat']
        values['zip_code'] = d['Location']['@zipcode']
        values['upcoming'] = int(d['Status'].get('@upcoming', 0))
        values['estate_type'] = d['@type']
        values['published_date'] = d['@insertedAt']
        values['modified_date'] = d['@modifiedAt']
        values['elevator'] = self._none_or_string_to_int(d['Elevator'])
        values['floor'] = d['Floor']
        values['balcony'] = d['Balcony']
        values['price'] = d['Economy']['@price']
        values['price_comment'] = d['Economy']['@priceComment']
        values['fee'] = d['Economy']['@fee']
        values['included_in_fee'] = d['Economy']['@includedInFee']
        values['not_included_in_fee'] = d['Economy']['@excludedInFee']
        values['currency'] = self._currency(json_item)
        values['estate_type_id'] = d['@typeId']
        values['rooms'] = d['Size']['@rooms']
        values['rooms_comment'] = d['Size']['@roomsInformation']
        values['min_bedrooms'] = d['Size']['@minBedrooms']
        values['max_bedrooms'] = d['Size']['@maxBedrooms']
        values['form_of_ownership'] = d['@formOfOwnership']
        values['apartment_number'] = d['ApartmentNumber']
        values['building_year'] = d['Built']
        values['window'] = d['Window']
        values['fireplace'] = d['Fireplace']
        values['television_and_broadband'] = d['TvInternet']
        values['association_part'] = d['Part']['Fee']
        values['no_showing_text'] = d['Showings']['@noShowingDefaultText']

        try:
            values['abroad'] = int(d['Status']['@abroad'])
        except KeyError:
            pass

        try:
            values['sold'] = int(d['@sold']) or int(d['Status']['@sold'])
        except KeyError:
            pass

        try:
            values['hidden'] = int(d['Status']['@hidden'])
        except KeyError:
            pass

        try:
            values['transfer_fee'] = d['Economy']['@transfer_fee']
        except KeyError:
            pass

        try:
            values['transfer_fee'] = d['Economy']['@pledge_fee']
        except KeyError:
            pass

        try:
            operating_costs = d['Economy']['OperatingCosts']

            values['monthly_operating_costs'] = operating_costs['@totalAmount']
            values['yearly_operating_costs'] = operating_costs['@totalAmountYear']
            values['operating_costs_people'] = operating_costs['@numberOfPersons']
            values['operating_costs_comment'] = operating_costs['@comment']

            values['operating_costs_text'] = self._operating_costs_text(
                json_item, values['currency'])
        except KeyError:
            pass

        if d['Building']:
            values['building_type'] = d['Building']['@buildingType']
            values['foundation'] = d['Building']['@foundation']
            values['framework'] = d['Building']['@framework']
            values['loft_floor'] = d['Building']['@loftFloor']
            values['facade'] = d['Building']['@facade']
            values['roof'] = d['Building']['@roof']
            values['chimney'] = d['Building']['@chimney']
            values['chimneypot'] = d['Building']['@chimneypot']
            values['energy_ventilation'] = d['Building']['@energyVentilation']
            values['municipal_water'] = d['Building']['@municipalWater']
            values['metalwork'] = d['Building']['@metalwork']
            values['heating_type'] = d['Building']['@heatingType']

        values['living_area'], values['living_area_unit'] = \
            self._get_area_and_unit_from_json('Boarea', d)
        values['non_living_area'], values['non_living_area_unit'] = \
            self._get_area_and_unit_from_json('Biarea', d)
        values['lands_area'], values['lands_area_unit'] = \
            self._get_area_and_unit_from_json('Tomtarea', d)
        values['general_area'], values['general_area_unit'] = \
            self._get_area_and_unit_from_json('Yta', d)

        values['description'] = self._get_description('Beskrivning', d)
        values['rooms_description'] = self._get_description('Rumsbeskrivning', d)
        values['association_description'] = self._get_description('Förening', d)
        values['detailed_description'] = self._get_description('Detaljerad information', d)
        values['district_description'] = self._get_district_description(d)

        values['in_central_stockholm'] = self._in_central_stockholm(d['Location']['@district'])
        values['is_priority_type'] = d['@typeId'] in settings.PRIORITY_TYPES

        try:
            instance = self.model.objects.get(fasad_id=fasad_id)
            created = False
        except self.model.DoesNotExist:
            instance = None
            created = True

        if published is not None:
            values['published'] = published
        else:
            if instance:
                values['published'] = instance.published

        form = RealEstateForm(values, instance=instance)
        if form.is_valid():
            real_estate = form.save()
            Image.objects.update_or_create_from_json(
                real_estate=real_estate, json_item=d)
            Document.objects.update_or_create_from_json(
                real_estate=real_estate, json_item=d)
            Media.objects.update_or_create_from_json(
                real_estate=real_estate, json_item=d)
            Association.objects.update_or_create_from_json(
                real_estate=real_estate, json_item=d)
            Showing.objects.update_or_create_from_json(
                real_estate=real_estate, json_item=d)
            Bid.objects.update_or_create_from_json(
                real_estate=real_estate, json_item=d)

            return (real_estate, created)
        else:
            raise ValidationError(form.errors.as_json())

    @classmethod
    def _get_realtor_pk(cls, json_item):
        """
        May throw Realtor.DoesNotExist.
        """
        from .models import Realtor
        realtor_list = utils.ensure_list(json_item['Realtors']['Realtor'])
        fasad_id = next(
            (realtor for realtor in realtor_list if realtor['@type'] == 'Ansvarig mäklare'),
            realtor_list[0]
        )['@id']

        realtor_pk = Realtor.objects.values('pk').get(fasad_id=fasad_id)['pk']

        return realtor_pk

    @classmethod
    def _get_area_and_unit_from_json(cls, area_type, json_item):
        result_area = '0'
        result_area_unit = ''

        areas = utils.ensure_list(json_item['Size']['Areas']['Area'])
        for area in areas:
            if area['@type'] == area_type:
                result_area = area['@size']
                result_area_unit = area['@unit']

        return result_area, result_area_unit

    @staticmethod
    def _currency(json_item):
        return json_item['Economy']['@currency']

    @classmethod
    def _operating_costs_text(cls, json_item, currency):
        costs= utils.ensure_list(json_item['Economy']['OperatingCosts']['OperatingCost'])

        costs_formatted = ['{alias} {amount} {currency}/mån'.format(
            alias=cost['@alias'],
            amount=utils.currency_format(cost['@amount']),
            currency=currency,
        ) for cost in costs]

        operating_costs_text = ', '.join(costs_formatted).lower()

        return operating_costs_text

    @staticmethod
    def _in_central_stockholm(district_name):
        simplified_district_name = re.split('[^A-Za-zÅÄÖåäö]', district_name)[0].lower()
        return simplified_district_name in [
            'södermalm', 'östermalm', 'vasastan', 'vasastaden', 'kungsholmen', 'birkastan']

    def _get_description(self, name, json_item):
        if json_item['Descriptions'] is None:
            return ''

        descriptions = utils.ensure_list(json_item['Descriptions']['Description'])
        result = ''

        for description in descriptions:
            if description['@category'] == name:
                if description['Title'] == 'Gemensamhetsanläggningar':
                    header = 'Servitut och gemensamhetsanläggningar'
                    content = '<p>' + json_item['Servitudes']['@comment'] + '</p>'
                else:
                    header = description['Title']
                    content = description['Content']

                result += '{header}{content}'.format(
                    header='<h3>' + header + '</h3>' if header else '',
                    content=content)

        return result

    def _get_district_description(self, json_item):
        if json_item['Descriptions'] is None:
            return ''

        descriptions = utils.ensure_list(json_item['Descriptions']['Description'])
        result = ''

        for description in descriptions:
            if description['@category'] == "Områdesbeskrivning":
                content = description['Content']

                result = content

        return result


class FasAdAssociationManager(BaseManager):

    def update_or_create_from_json(self, real_estate, json_item):
        if json_item['Association']:
            association, _ = self.model.objects.update_or_create(
                organization_number=json_item['Association']['@organizationNo'],
                defaults={
                    'name': json_item['Association']['@name'],
                    'organization_number': json_item['Association']['@organizationNo'],
                    'genuine': json_item['Association']['@genuine'],
                }
            )


class FasAdShowingManager(BaseManager):

    def update_or_create_from_json(self, real_estate, json_item):
        # TODO: Handle marking for deletion instead
        self.filter(real_estate=real_estate).delete()

        try:
            showings = utils.ensure_list(json_item['Showings']['Showing'])
            for showing in showings:
                start_date = self._parse_datetime(showing['@startDate'])
                end_date = self._parse_datetime(showing['@endDate'])

                self.create(
                    real_estate=real_estate,
                    start_date=start_date,
                    end_date=end_date,
                    comment=showing['@comment'],
                )
        except KeyError:
            pass  # There are no showings

    def _parse_datetime(self, datetime_string):
        if len(datetime_string) == 10:
            datetime_string += ' 00:00:00'

        try:
            return timezone.make_aware(parse_datetime(datetime_string))
        except:
            return None


class FasAdBidManager(BaseManager):

    def update_or_create_from_json(self, real_estate, json_item):
        # TODO: Handle marking for deletion instead
        self.filter(real_estate=real_estate).delete()

        try:
            bids = utils.ensure_list(json_item['Bids']['Bid'])
            for bid in bids:
                try:
                    bid_datetime = timezone.make_aware(parse_datetime(bid['@date']))
                except:
                    bid_datetime = None

                self.create(
                    real_estate=real_estate,
                    amount=bid['@amount'],
                    bidder=bid['@bidder'],
                    bid_datetime=bid_datetime,
                )
        except KeyError:
            pass  # There are no bids


class FasAdDocumentManager(BaseManager):

    def update_or_create_from_json(self, real_estate, json_item):
        # TODO: Handle marking for deletion instead
        self.filter(real_estate=real_estate).delete()
        documents = []

        try:
            documents.extend(utils.ensure_list(
                json_item['ObjectDocuments']['ObjectDocument']))
        except TypeError:
            pass

        try:
            documents.extend(utils.ensure_list(
                json_item['ApartmentCooperativeDocuments']['ApartmentCooperativeDocument']))
        except TypeError:
            pass

        for document in documents:
            self.create(
                real_estate=real_estate,
                name=document['@alias'],
                file_size=document['@filesize'],
                url=document['@href'],
            )


class FasAdMediaManager(BaseManager):

    def update_or_create_from_json(self, real_estate, json_item):
        # TODO: Handle marking for deletion instead
        self.filter(real_estate=real_estate).delete()

        try:
            medias = utils.ensure_list(json_item['Media']['MediaItem'])

            for index, media in enumerate(medias):
                self.create(
                    real_estate=real_estate,
                    name=media['@alias'],
                    url=media['@url'],
                    priority=index,
                )
        except TypeError:
            pass  # There's no media


class FasAdImageManager(BaseManager):

    def update_or_create_from_json(self, real_estate, json_item):
        images = self._images(json_item)
        fasad_ids = []

        for index, image in enumerate(images):
            fasad_id = int(image['@id'])
            fasad_ids.append(fasad_id)

            defaults = dict()
            defaults['priority'] = index
            defaults['real_estate'] = real_estate
            defaults['comment'] = self._comment(image)
            defaults['portrait'] = self._portrait(image)
            defaults['category'] = image['@category']
            defaults['floorplan'] = (defaults['category'] == 'Planlösningar')

            variant = self._variant(image)
            defaults['width'] = image[variant]['@width']
            defaults['height'] = image[variant]['@height']
            defaults['file_size'] = image[variant]['@size']
            defaults['source_url'] = image[variant]['@path']

            image_instance, _ = self.model.objects.update_or_create(
                 fasad_id=fasad_id,
                 defaults=defaults)

        self._mark_panorama(real_estate)
        self._set_landscape_index(real_estate)

        self.filter(real_estate=real_estate).exclude(fasad_id__in=fasad_ids).delete()

    def _mark_panorama(self, real_estate):
        self.filter(real_estate=real_estate).update(panorama=False)
        image = self.filter(real_estate=real_estate).order_by('priority').last()

        if image is not None:
            image.panorama = True
            image.save()

    def _images(self, json_item):
        if json_item['Images'] is None:
            return []
        else:
            return utils.ensure_list(json_item['Images']['Image'])

    def _portrait(self, image_json):
        variant = self._variant(image_json)
        height = int(image_json[variant]['@height'])
        width = int(image_json[variant]['@width'])
        is_portrait = (height > width)
        return is_portrait

    def _variant(self, image_json):
        if 'Highres' in image_json:
            return 'Highres'
        else:
            return 'Large'

    def _set_landscape_index(self, real_estate):
        landscape_index = 0
        for image in self.filter(real_estate=real_estate):
            if image.portrait:
                image.landscape_index = -1
            else:
                image.landscape_index = landscape_index
                landscape_index += 1
            image.save()

    def _comment(self, image_json):
        comment = image_json['@text']

        if self._is_valid_comment(comment):
            return comment
        else:
            return ''

    def _is_valid_comment(self, comment):
        is_invalid = re.match('\d{8} \d{3}', comment)
        return not is_invalid

    def list_images(self):
        return self.filter(panorama=False)

    def list_image(self):
        return self.first()

    def panorama_image(self):
        return self.filter(panorama=True).last()

FasAdRealEstateManager = FasAdRealEstateBaseManager.from_queryset(RealEstateQuerySet)
