from django.db import models
from django.db import IntegrityError
from django.core.urlresolvers import reverse
from django.utils import numberformat
from django.conf import settings

from .managers import (FasAdRealtorManager, FasAdRealEstateManager, FasAdImageManager,
                       FasAdDocumentManager, FasAdMediaManager, FasAdAssociationManager,
                       FasAdShowingManager, FasAdBidManager,)
from .utils import slugify


class Association(models.Model):
    name = models.CharField(max_length=100)
    organization_number = models.CharField(max_length=20)
    genuine = models.NullBooleanField(default=True)

    objects = FasAdAssociationManager()

    def __str__(self):
        return self.name


class Realtor(models.Model):

    class Meta:
        ordering = ['priority']

    fasad_id = models.IntegerField()
    first_name = models.CharField(max_length=100)
    last_name = models.CharField(max_length=100)
    title = models.CharField(max_length=100)
    email = models.CharField(max_length=100)
    phone = models.CharField(max_length=30)
    image_url = models.URLField(max_length=200)
    public = models.BooleanField(default=False)
    priority = models.IntegerField()

    objects = FasAdRealtorManager()

    def __str__(self):
        return self.name()

    def name(self):
        return '{} {}'.format(self.first_name, self.last_name)


class RealEstate(models.Model):

    fasad_id = models.IntegerField(primary_key=True)
    slug = models.SlugField(blank=True, unique=True)
    published = models.BooleanField(default=False)
    address = models.CharField(max_length=200, blank=True)
    country = models.CharField(max_length=100, blank=True)
    city = models.CharField(max_length=100, blank=True)
    district = models.CharField(max_length=100, blank=True)
    zip_code = models.CharField(max_length=20, blank=True)
    latitude = models.FloatField(blank=True, null=True)
    longitude = models.FloatField(blank=True, null=True)
    currency = models.CharField(max_length=10, blank=True)
    price = models.IntegerField(blank=True, null=True)
    price_comment = models.CharField(max_length=200, blank=True)
    fee = models.IntegerField(blank=True, null=True)
    included_in_fee = models.CharField(max_length=200, blank=True)
    not_included_in_fee = models.CharField(max_length=200, blank=True)
    apartment_number = models.CharField(max_length=100, blank=True)
    rooms = models.CharField(max_length=10, blank=True)
    rooms_comment = models.CharField(max_length=200, blank=True)
    min_bedrooms = models.IntegerField(blank=True, null=True)
    max_bedrooms = models.IntegerField(blank=True, null=True)
    living_area = models.FloatField(blank=True, null=True)
    living_area_unit = models.CharField(max_length=20, blank=True)
    non_living_area = models.FloatField(blank=True, null=True)
    non_living_area_unit = models.CharField(max_length=20, blank=True)
    lands_area = models.FloatField(blank=True, null=True)
    lands_area_unit = models.CharField(max_length=20, blank=True)
    general_area = models.FloatField(blank=True, null=True)
    general_area_unit = models.CharField(max_length=20, blank=True)
    form_of_ownership = models.CharField(max_length=200)
    status = models.CharField(max_length=100, blank=True)
    sold = models.BooleanField(default=False)
    estate_type = models.CharField(max_length=100, blank=True)
    estate_type_id = models.IntegerField(null=True, blank=True)
    balcony = models.CharField(max_length=200, blank=True, null=True)
    floor = models.CharField(max_length=200, blank=True)
    elevator = models.NullBooleanField(default=False)
    abroad = models.BooleanField(default=False)
    hidden = models.BooleanField(default=False)
    in_central_stockholm = models.BooleanField(default=False)
    is_priority_type = models.BooleanField(default=False)
    upcoming = models.BooleanField(default=False)
    building_type = models.CharField(max_length=200, blank=True)
    building_year = models.CharField(max_length=200, blank=True)
    heating_type = models.CharField(max_length=200, blank=True)
    television_and_broadband = models.CharField(max_length=200, blank=True)
    association_part = models.CharField(max_length=100, blank=True)
    no_showing_text = models.CharField(max_length=200, blank=True)

    foundation = models.CharField(max_length=200, blank=True)
    framework = models.CharField(max_length=200, blank=True)
    loft_floor = models.CharField(max_length=200, blank=True)
    facade = models.CharField(max_length=200, blank=True)
    roof = models.CharField(max_length=200, blank=True)
    chimney = models.CharField(max_length=200, blank=True)
    chimneypot = models.CharField(max_length=200, blank=True)
    energy_ventilation = models.CharField(max_length=200, blank=True)
    municipal_water = models.CharField(max_length=200, blank=True)
    metalwork = models.CharField(max_length=200, blank=True)
    window = models.CharField(max_length=200, blank=True)
    fireplace = models.CharField(max_length=200, blank=True)

    transfer_fee = models.IntegerField(blank=True, null=True)
    pledge_fee = models.IntegerField(blank=True, null=True)

    monthly_operating_costs = models.IntegerField(blank=True, null=True)
    yearly_operating_costs = models.IntegerField(blank=True, null=True)
    operating_costs_people = models.IntegerField(blank=True, null=True)
    operating_costs_text = models.TextField(blank=True)
    operating_costs_comment = models.TextField(blank=True)

    description = models.TextField(blank=True)
    rooms_description = models.TextField(blank=True)
    district_description = models.TextField(blank=True)
    association_description = models.TextField(blank=True)
    detailed_description = models.TextField(blank=True)

    published_date = models.DateTimeField()
    modified_date = models.DateTimeField()
    # last_synchronized = models.DateTimeField(blank=True, null=True)

    association = models.ForeignKey(Association, blank=True, null=True)
    realtor = models.ForeignKey(Realtor, on_delete=models.SET_NULL, blank=True, null=True)

    objects = FasAdRealEstateManager()

    def __str__(self):
        return '{} ({})'.format(self.address, self.fasad_id)

    def save(self, **kwargs):
        self.slug = slugify(self.address)

        if RealEstate.objects.filter(slug=self.slug).exclude(pk=self.pk).exists():
            self.slug = '{}-{}'.format(self.slug, self.fasad_id)

        super(RealEstate, self).save(**kwargs)

    def panorama_image(self):
        return self.image_set.panorama_image()

    def list_image(self):
        return self.image_set.list_image()

    def list_images(self):
        return self.image_set.list_images()


class Showing(models.Model):

    class Meta:
        ordering = ['start_date']

    start_date = models.DateTimeField(null=True, blank=True)
    end_date = models.DateTimeField(null=True, blank=True)
    comment = models.CharField(max_length=200)

    real_estate = models.ForeignKey(RealEstate)

    objects = FasAdShowingManager()


class Bid(models.Model):

    class Meta:
        ordering = ['-bid_datetime', '-amount']

    amount = models.IntegerField()
    bidder = models.CharField(max_length=200)
    bid_datetime = models.DateTimeField(null=True, blank=True)

    real_estate = models.ForeignKey(RealEstate)

    objects = FasAdBidManager()


class Document(models.Model):
    name = models.CharField(max_length=100)
    url = models.URLField(max_length=200)
    file_size = models.IntegerField()

    real_estate = models.ForeignKey(RealEstate)

    objects = FasAdDocumentManager()


class Media(models.Model):

    class Meta:
        ordering = ['priority']

    name = models.CharField(max_length=200)
    url = models.URLField(max_length=200)
    priority = models.IntegerField()

    real_estate = models.ForeignKey(RealEstate)

    objects = FasAdMediaManager()


class Image(models.Model):

    class Meta:
        ordering = ['panorama', 'floorplan', 'priority']

    fasad_id = models.IntegerField()
    priority = models.IntegerField()
    category = models.CharField(max_length=100)
    comment = models.CharField(max_length=200)
    width = models.IntegerField()
    height = models.IntegerField()
    landscape_index = models.IntegerField(null=True, blank=True, default=None)
    portrait = models.BooleanField(default=False)
    panorama = models.BooleanField(default=False)
    floorplan = models.BooleanField(default=False)
    file_size = models.IntegerField()
    source_url = models.URLField(max_length=255)

    real_estate = models.ForeignKey(RealEstate)

    objects = FasAdImageManager()

    def relative_source_url(self):
        return self.source_url[37:]

    def list_url(self):
        return settings.RESIZE_LIST_URL.format(url=self.source_url)

    def fullscreen_url(self):
        return settings.RESIZE_FULLSCREEN_URL.format(url=self.source_url)

    def panorama_url(self):
        return settings.RESIZE_PANORAMA_URL.format(url=self.source_url)

    def resized_width(self):
        return settings.LIST_WIDTH

    def resized_height(self):
        return int(self.height / self.width * settings.LIST_WIDTH)
