import os
import json
import unittest
import requests
from requests import Response
from requests.exceptions import HTTPError

from unittest.mock import patch

from fasad.api import FasAd


class FasAdTest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        cls.fasad = FasAd(company_id=1, office_id=1)

    def _patch_xml(self, path, func, *args):
        with open(os.path.join('tests/fixtures/', path), mode='r', encoding='utf-8') as f:
            xml = f.read()

        with patch.object(self.fasad, '_get_xml_from_url', return_value=xml):
            result = func(*args)

        return result

    def _load_json(self, path):
        with open(os.path.join('tests/fixtures/', path), mode='r', encoding='utf-8') as f:
            return json.loads(f.read())

    # Realtors

    def test_realtors(self):
        """
        Should return all realtors as JSON.
        """
        realtors = self._patch_xml('realtors/normal.xml', self.fasad.realtors)
        expected_json = self._load_json('realtors/normal.json')

        self.assertEqual(len(realtors), 27)
        self.assertEqual(realtors, expected_json)

    def test_realtors_with_only_one_item(self):
        """
        Should return the single item without error.
        """
        realtors = self._patch_xml('realtors/single.xml', self.fasad.realtors)
        expected_json = self._load_json('realtors/single.json')

        self.assertEqual(len(realtors), 1)
        self.assertEqual(realtors, expected_json)

    def test_realtors_with_empty_list(self):
        """
        Should return zero realtors without error.
        """
        realtors = self._patch_xml('realtors/empty.xml', self.fasad.realtors)

        self.assertEqual(len(realtors), 0)

    # Real estates

    def test_real_estates(self):
        """
        Should return all real estates as JSON.
        """
        real_estates = self._patch_xml('real_estates/normal.xml', self.fasad.real_estates)
        expected_json = self._load_json('real_estates/normal.json')

        self.assertEqual(len(real_estates), 53)
        self.assertEqual(real_estates, expected_json)

    def test_real_estates_with_only_one_item(self):
        """
        Should return the single item without error.
        """
        real_estates = self._patch_xml('real_estates/single.xml', self.fasad.real_estates)
        expected_json = self._load_json('real_estates/single.json')

        self.assertEqual(len(real_estates), 1)
        self.assertEqual(real_estates, expected_json)

    def test_real_estates_with_empty_list(self):
        """
        Should return zero real estates without error.
        """
        real_estates = self._patch_xml('real_estates/empty.xml', self.fasad.real_estates)

        self.assertEqual(len(real_estates), 0)

    # def test__get_modified_real_estate_ids(self):
    #     with patch.object(self.manager, '_get_xml_from_url', return_value=self.list_short_xml):
    #         modified_ids = self.manager._get_modified_fasad_ids()

    #     self.assertEqual(modified_ids, ['422143', '199014'])

    # Real estate

    def test_real_estate(self):
        """
        Should return a JSON representation of a single real estate.
        """
        fasad_id = 244747
        real_estate = self._patch_xml('real_estate/{}.xml'.format(fasad_id),
                                      self.fasad.real_estate, fasad_id)
        expected_json = self._load_json('real_estate/{}.json'.format(fasad_id))

        self.assertEqual(real_estate, expected_json)

    def test_real_estate_nonexisting_fasad_id(self):
        """
        Should raise
        """
        response = Response()
        response.status_code = 404

        with patch.object(requests, 'get', return_value=response):
            with self.assertRaises(HTTPError):
                self.fasad.real_estate(1)
