import json
import unittest

from django.test import TestCase

from fasad.managers import FasAdImageManager
from fasad.models import Image
from fasad import factories


class FasAdImageManagerTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdImageManager()
        cls.manager.model = Image
        cls.real_estate = factories.RealEstate()

        cls.bids_zero = {"Bids": {}}

        cls.bids_single = {"Bids": {"Bid": [{
               "@amount": "100",
               "@bidder": "Budgivare 1",
        }]}}

        cls.bids_multiple = {"Bids": {"Bid": [
            {
                "@amount": "100",
                "@bidder": "Budgivare 1",
            },
            {
                "@amount": "200",
                "@bidder": "Budgivare 2",
            },
        ]}}


class ListImagesTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdImageManager()
        cls.manager.model = Image

    def test_excludes_panorama(self):
        """
        It excludes the panorama image as that's the only image that shouldn't be listed.
        """
        list_image = factories.Image(panorama=False)
        panorama_image = factories.Image(panorama=True)

        list_images = self.manager.list_images()

        self.assertEqual(list(list_images), [list_image])

    def test_returns_all_if_no_panorama(self):
        """
        It returns all images if there's no panorama.
        """
        first_image = factories.Image(panorama=False)
        second_image = factories.Image(panorama=False)

        list_images = self.manager.list_images()

        self.assertEqual(list(list_images), [first_image, second_image])


class ListImageTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdImageManager()
        cls.manager.model = Image

    def test_with_different_types(self):
        """
        It returns the first image that is not a floorplan or panorama.
        """
        floorplan_image = factories.Image(floorplan=True)
        panorama_image = factories.Image(panorama=True)
        list_image = factories.Image()
        second_image = factories.Image()

        self.assertEqual(self.manager.list_image(), list_image)

    def test_with_only_floorplan_and_panorama(self):
        """
        It returns a floorplan if there are no "normal" images.
        """
        panorama = factories.Image(panorama=True)
        floorplan = factories.Image(floorplan=True)

        self.assertEqual(self.manager.list_image(), floorplan)

    def test_with_only_panorama(self):
        """
        It returns the panorama if there are no floorplans or "normal" images.
        """
        panorama = factories.Image(panorama=True)

        self.assertEqual(self.manager.list_image(), panorama)

    def test_without_images(self):
        """
        It returns None if there are no images at all.
        """
        list_image = self.manager.list_image()

        self.assertEqual(list_image, None)


class PanoramaImageTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdImageManager()
        cls.manager.model = Image

    def test_returns_panorama(self):
        """
        It returns the panorama even if there are other images.
        """
        list_image = factories.Image(panorama=False)
        panorama_image = factories.Image(panorama=True)

        self.assertEqual(self.manager.panorama_image(), panorama_image)

    def test_with_multiple_panoramas(self):
        """
        It returns only the last panorama if there are more than one (which shouldn't happen).
        """
        first_panorama = factories.Image(panorama=True)
        second_panorama = factories.Image(panorama=True)

        self.assertEqual(self.manager.panorama_image(), second_panorama)


class UpdateOrCreateFromJsonTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdImageManager()
        cls.manager.model = Image
        cls.real_estate = factories.RealEstate()

    def test_happy_path(self):
        """
        It persists the images with a relation to the given real estate.
        """
        with open('tests/fixtures/item.json', encoding='utf-8') as f:
            json_item = json.loads(f.read())['Item']

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        image = Image.objects.get(fasad_id=1780909)
        self.assertEqual(image.real_estate, self.real_estate)
        self.assertEqual(image.category, 'Övrigt')
        self.assertEqual(image.priority, 0)
        self.assertEqual(image.comment, '')
        self.assertEqual(image.width, 900)
        self.assertEqual(image.height, 600)
        self.assertEqual(image.landscape_index, 0)
        self.assertEqual(image.panorama, False)
        self.assertEqual(image.portrait, False)
        self.assertEqual(image.floorplan, False)
        self.assertEqual(image.file_size, 249269)
        self.assertEqual(image.source_url,
                         'https://bilder.fasad.eu/192/400128/340639/large/1780909.jpg')

    def test_portrait(self):
        """
        It correctly marks portrait images as portrait.
        """
        with open('tests/fixtures/item.json', encoding='utf-8') as f:
            json_item = json.loads(f.read())['Item']

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        portrait_image = Image.objects.get(fasad_id=1780915)
        landscape_image = Image.objects.get(fasad_id=1780909)
        self.assertEqual(portrait_image.portrait, True)
        self.assertEqual(landscape_image.portrait, False)

    def test_panorama(self):
        """
        It correctly marks the last image as panorama.
        """
        with open('tests/fixtures/item.json', encoding='utf-8') as f:
            json_item = json.loads(f.read())['Item']

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        panorama_image = Image.objects.get(panorama=True)
        self.assertEqual(panorama_image.fasad_id, 1792929)

    def test_with_zero_images(self):
        """
        It simply returns if there are no images.
        """
        json_item = {"Images": None}

        self.assertFalse(self.manager.exists())

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        self.assertFalse(self.manager.exists())


class PortraitTest(unittest.TestCase):
    def test_with_portrait_image(self):
        image_json = {
            "Highres": {
                "@height": "200",
                "@width": "100"
            }
        }

        portrait = FasAdImageManager()._portrait(image_json)

        self.assertEqual(portrait, True)

    def test_with_landscape_image(self):
        image_json = {
            "Highres": {
                "@height": "100",
                "@width": "200"
            }
        }

        portrait = FasAdImageManager()._portrait(image_json)

        self.assertEqual(portrait, False)

    def test_with_square_image(self):
        image_json = {
            "Highres": {
                "@height": "100",
                "@width": "100"
            }
        }

        portrait = FasAdImageManager()._portrait(image_json)

        self.assertEqual(portrait, False)


class CommentTest(unittest.TestCase):
    def test_with_valid_comment(self):
        """
        It returns the comment.
        """
        image_json = {"@text": "Valid comment"}

        comment = FasAdImageManager()._comment(image_json)

        self.assertEqual(comment, 'Valid comment')

    def test_with_invalid_comment(self):
        """
        It returns an empty string.
        """
        image_json = {"@text": "20131101 123"}

        comment = FasAdImageManager()._comment(image_json)

        self.assertEqual(comment, '')


class MarkPanoramaTest(TestCase):
    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdImageManager()
        cls.manager.model = Image
        cls.real_estate = factories.RealEstate()

    def test_marks_last_image(self):
        """
        It marks the last image as panorama and unmarks all others.
        """
        old_panorama = factories.Image(
            real_estate=self.real_estate,
            fasad_id=1780939,
            panorama=True
        )
        with open('tests/fixtures/item.json', encoding='utf-8') as f:
            json_item = json.loads(f.read())['Item']

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        panorama_image = self.manager.get(panorama=True)
        old_panorama.refresh_from_db()
        self.assertEqual(panorama_image.fasad_id, 1792929)
        self.assertEqual(old_panorama.panorama, False)

    def test_multiple_real_estates(self):
        """
        It does not unmark other real estates' panoramas.
        """
        other_estates_panorama = factories.Image(panorama=True)
        with open('tests/fixtures/item.json', encoding='utf-8') as f:
            json_item = json.loads(f.read())['Item']

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        panorama_images = self.manager.filter(panorama=True)
        new_panorama = Image.objects.get(fasad_id=1792929)
        self.assertEqual(list(panorama_images), [other_estates_panorama, new_panorama])


class IsValidComment(unittest.TestCase):
    def test_with_valid_comment(self):
        self.assertTrue(FasAdImageManager()._is_valid_comment('Valid comment'))

    def test_with_invalid_comment(self):
        self.assertFalse(FasAdImageManager()._is_valid_comment('20131113 332'))


class Images(unittest.TestCase):
    def test_with_multiple_images(self):
        """
        It returns a list of images.
        """
        json_item = {
            "Images": {
                "Image": [
                    'first_image',
                    'second_image'
                ]
            }
        }

        images = FasAdImageManager()._images(json_item)

        self.assertEqual(images, ['first_image', 'second_image'])

    def test_with_one_image(self):
        """
        It turns a single image into a list.
        """
        json_item = {
            "Images": {
                "Image": 'single_image'
            }
        }

        images = FasAdImageManager()._images(json_item)

        self.assertEqual(images, ['single_image'])

    def test_with_zero_images(self):
        """
        It returns an empty list when there are no images.
        """
        json_item = {"Images": None}

        images = FasAdImageManager()._images(json_item)

        self.assertEqual(images, [])


class VariantTest(TestCase):
    def test_default(self):
        """
        It defaults to 'Large'.
        """
        image_json = {}

        variant = FasAdImageManager()._variant(image_json)

        self.assertEqual(variant, 'Large')

    def test_highres(self):
        """
        It returns 'Highres' if highres version is available.
        """
        image_json = {
            "Highres": {},
            "Large": {}
        }

        variant = FasAdImageManager()._variant(image_json)

        self.assertEqual(variant, 'Highres')
