from unittest.mock import patch
import unittest
import json

from django.utils import timezone
from django.test import TestCase
from django.core.urlresolvers import reverse

from fasad import factories
from fasad.models import Association, Realtor, RealEstate, Image


class AssociationTest(unittest.TestCase):

    def test__str__(self):
        association = Association(name='Foobar', organization_number='123456-7890')
        self.assertEqual(str(association), 'Foobar')


class RealtorTest(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.realtor = Realtor(first_name='Foo', last_name='Bar')

    def test___str__(self):
        self.assertEqual(str(self.realtor), 'Foo Bar')

    def test_name(self):
        self.assertEqual(self.realtor.name(), 'Foo Bar')


class RealEstateTest(TestCase):

    def setUp(self):
        realtor = Realtor(pk=1, fasad_id=1)
        self.real_estate = RealEstate(
            fasad_id=1,
            address='Foobar',
            realtor=realtor,
            published_date=timezone.now(),
            modified_date=timezone.now(),
        )

    def test__str__(self):
        self.assertEqual(str(self.real_estate), 'Foobar (1)')

    def test_save_slugify(self):
        """
        Address should be slugified on save.
        """
        self.assertEqual(self.real_estate.slug, '')
        self.real_estate.save()
        self.assertEqual(self.real_estate.slug, 'foobar')

    def test_save_slugify_repeatedly(self):
        """
        Address should be re-slugified on save.
        """
        self.assertEqual(self.real_estate.slug, '')
        self.real_estate.save()
        self.assertEqual(self.real_estate.slug, 'foobar')
        self.real_estate.address = 'Fizzbuzz'
        self.real_estate.save()
        self.assertEqual(self.real_estate.slug, 'fizzbuzz')

    def test_save_slugify_unique(self):
        """
        Should generate a unique slug if the address is not unique.
        """
        address = 'Foo bar'

        factories.RealEstate(address=address)
        real_estate = factories.RealEstate(fasad_id=90, address=address)

        self.assertEqual(real_estate.slug, 'foo-bar-90')

    def test_save_slugify_should_not_uniquify_unless_needed(self):
        """
        Should not try to generate a unique slug unless needed.
        """
        real_estate = factories.RealEstate(address='Foo bar')

        self.assertEqual(real_estate.slug, 'foo-bar')
        real_estate.save()
        self.assertEqual(real_estate.slug, 'foo-bar')


class ImageTest(TestCase):

    def setUp(self):
        self.image = factories.Image.build(
            source_url='http://example.com/image.jpg',
            width=300,
            height=200,
        )

    def test_default_ordering(self):
        real_estate = factories.RealEstate()
        panorama = factories.Image(panorama=True, real_estate=real_estate)
        floorplan = factories.Image(floorplan=True, real_estate=real_estate)
        priority1 = factories.Image(priority=1, real_estate=real_estate)
        priority2 = factories.Image(priority=2, real_estate=real_estate)

        self.assertEqual(
            [priority1, priority2, floorplan, panorama],
            list(Image.objects.all()))

    def test_list_url(self):
        """
        Should return a resize URL.
        """
        self.assertEqual(self.image.list_url(),
                         'http://process.fasad.eu/rimage.php?url='
                         'http://example.com/image.jpg&m=resize&w=952&i=1')

    def test_fullscreen_url(self):
        """
        Should return a fullscreen resize URL.
        """
        self.assertEqual(self.image.fullscreen_url(),
                         'http://process.fasad.eu/rimage.php?url='
                         'http://example.com/image.jpg&m=resize&w=1920&i=1')

    def test_panorama_url(self):
        """
        Should return a panorama resize URL.
        """
        self.assertEqual(self.image.panorama_url(),
                         'http://process.fasad.eu/rimage.php?url='
                         'http://example.com/image.jpg&m=strict&w=1920&h=750&i=1')

    def test_resized_width(self):
        self.assertEqual(self.image.resized_width(), 952)

    def test_resized_height(self):
        self.assertEqual(self.image.resized_height(), 634)
