import json
from unittest.mock import patch

from django.test import TestCase

from fasad.managers import FasAdRealEstateManager
from fasad.models import RealEstate, Realtor


class FasAdRealEstateManagerTest(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdRealEstateManager()
        cls.manager.model = RealEstate

        with open('tests/fixtures/realtors/normal.json', mode='r', encoding='utf-8') as f:
            realtors_json = json.loads(f.read())

        with patch.object(Realtor.objects.fasad, 'realtors', return_value=realtors_json):
            Realtor.objects.synchronize()

    def test_synchronize_saves_all_details(self):
        pass

    def test_saves_general_area(self):
        with open('tests/fixtures/real_estate/244747.json', mode='r', encoding='utf-8') as f:
            real_estate_json = json.loads(f.read())

        real_estate_json['Size']['Areas'] = {"Area": {
            "@type": "Yta",
            "@size": "50",
            "@unit": "Kvm",
        }}

        with patch.object(self.manager.fasad, 'real_estate', return_value=real_estate_json):
            self.manager.update_or_create_from_fasad(244747)

        real_estate = RealEstate.objects.get(fasad_id=244747)
        self.assertEqual(real_estate.general_area, 50)
        self.assertEqual(real_estate.general_area_unit, 'Kvm')

    def test_synchronize_does_not_delete_real_estates(self):
        """
        Should not delete real estates that were not part of the synchronization.
        """
        pass
        # old_real_estate = factories.RealEstate()

        # self.assertTrue(self.manager.exists())

        # with patch.object(self.manager.fasad, 'real_estates', return_value=[]):
        #     self.manager.synchronize()

        # self.assertTrue(self.manager.exists())

    def test_synchronize_updates_existing_real_estates(self):
        pass

    def test__operating_costs_text(self):
        """
        Should format the operating costs and concatenate them.
        """
        json_item = {
            "Economy": {
                "OperatingCosts": {
                    "OperatingCost": [
                        {"@alias": "Värme", "@amount": "891", "@amountYear": "10690"},
                        {"@alias": "El", "@amount": "5000", "@amountYear": "60000"}
                    ],
                }
            }
        }

        result = self.manager._operating_costs_text(json_item, currency="kr")

        self.assertEqual(result, 'värme 891 kr/mån, el 5 000 kr/mån')

    def test__currency(self):
        json_item = {"Economy": {"@currency": "eur"}}

        result = self.manager._currency(json_item)

        self.assertEqual(result, "eur")

    def test__in_central_stockholm_with_districts_outside_the_city(self):
        """
        Should return false if given district is outside central Stockholm.
        """
        districts_outside_city = ['Hägersten', 'Blåsut', 'Farsta strand']
        for district_name in districts_outside_city:
            self.assertFalse(self.manager._in_central_stockholm(district_name))

    def test__in_central_stockholm_with_district_name_not_in_central_stockholm(self):
        """
        Should return true if given district is in central Stockholm.
        """
        districts_in_central_stockholm = ['Södermalm', 'Vasastan', 'Vasastaden', 'Östermalm']
        for district_name in districts_in_central_stockholm:
            self.assertTrue(self.manager._in_central_stockholm(district_name))

    def test_get_area_and_unit_from_json_without_living_area(self):
        """
        Should return defaults if there's no area of the given type.
        """
        json_item = {"Size": {"Areas": {"Area": {"@type": "Tomtarea"}}}}
        result = self.manager._get_area_and_unit_from_json('Biarea', json_item)
        self.assertEqual(result, ('0', ''))

    def test_get_area_and_unit_from_json_with_only_living_area(self):
        """
        Should work if there's only one area type.
        """
        json_item = {"Size": {"Areas": {"Area": {
            "@type": "Boarea",
            "@size": "50",
            "@unit": "Kvm",
        }}}}

        result = self.manager._get_area_and_unit_from_json('Boarea', json_item)
        self.assertEqual(result, ('50', 'Kvm'))

    def test_get_area_and_unit_from_json_with_multiple_area_types(self):
        """
        Should work if there are multiple area types.
        """
        json_item = {"Size": {"Areas": {"Area": [
            {
               "@type": "Biarea",
               "@size": "18",
               "@unit": "Kvm",
            },
            {
               "@type": "Boarea",
               "@size": "114",
               "@unit": "Kvm",
            },
            {
               "@type": "Tomtarea",
               "@size": "303",
               "@unit": "Kvm",
            }
         ]}}}

        result = self.manager._get_area_and_unit_from_json('Biarea', json_item)
        self.assertEqual(result, ('18', 'Kvm'))

    # def test_update_or_create_from_fasad_with_new_valid_id(self):
    #     fasad_id = '340639'
    #     with patch.object(self.manager, '_get_xml_from_url', return_value=self.item_xml):
    #         real_estate, created = self.manager.update_or_create_from_fasad(fasad_id)

    #     self.assertTrue(created)
    #     self.assertEqual(real_estate.city, 'Stockholm')

    #     saved_real_estate = self.manager.model.objects.get(fasad_id=fasad_id)
    #     self.assertEqual(real_estate.city, saved_real_estate.city)

    # def test_update_or_create_from_fasad_with_old_valid_id(self):
    #     fasad_id = '340639'

    #     with patch.object(self.manager, '_get_xml_from_url', return_value=self.item_xml):
    #         real_estate, created = self.manager.update_or_create_from_fasad(fasad_id)

    #         self.assertEqual(real_estate.city, 'Stockholm')
    #         real_estate.city = 'New york'
    #         real_estate.save()

    #         real_estate, created = self.manager.update_or_create_from_fasad(fasad_id)
    #         self.assertEqual(real_estate.city, 'Stockholm')
    #         self.assertFalse(created)



