import json
from unittest.mock import patch

from django.test import TestCase

from fasad.models import Realtor, RealEstate
from fasad.managers import FasAdRealtorManager
from fasad import factories


class FasAdRealtorManagerTest(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdRealtorManager()
        cls.manager.model = Realtor

        with open('tests/fixtures/realtors/normal.json', mode='r', encoding='utf-8') as f:
            cls.realtors_json = json.loads(f.read())

    def test_synchronize_saves_realtor_details(self):
        with patch.object(self.manager.fasad, 'realtors', return_value=self.realtors_json):
            self.manager.synchronize()

        realtor = self.manager.get(fasad_id=1)
        self.assertEqual(realtor.first_name, 'Anders')
        self.assertEqual(realtor.last_name, 'Spetz')
        self.assertEqual(realtor.title, '')
        self.assertEqual(realtor.email, 'anders.spetz@prek.se')
        self.assertEqual(realtor.phone, '070-267 79 53')
        self.assertEqual(realtor.image_url, 'https://images02.fasad.eu/1/user/1.jpg')
        self.assertEqual(realtor.public, True)

        nonpublic_realtor = self.manager.get(fasad_id=618)
        self.assertEqual(nonpublic_realtor.public, False)

    def test_synchronize_saves_multiple_realtors(self):
        """
        Should fetch and save all realtors from FasAd.
        """
        with patch.object(self.manager.fasad, 'realtors', return_value=self.realtors_json):
            self.manager.synchronize()

        self.assertEqual(self.manager.count(), 27)

    def test_synchronize_updates_existing_realtors(self):
        """
        Should update existing realtors with new information from FasAd.
        """
        existing_realtor = factories.Realtor(fasad_id=1, first_name='Existing')

        with patch.object(self.manager.fasad, 'realtors', return_value=self.realtors_json):
            self.manager.synchronize()

        self.assertEqual(self.manager.get(fasad_id=1).first_name, 'Anders')

    def test_synchronize_deletes_old_realtors(self):
        """
        Should delete realtors that don't exist in FasAd anymore.
        """
        old_realtor = factories.Realtor(fasad_id=1)

        with patch.object(self.manager.fasad, 'realtors', return_value=[]):
            self.manager.synchronize()

        self.assertFalse(self.manager.filter(fasad_id=1).exists())

    def test_synchronize_handles_zero_realtors(self):
        """
        Should not fail if there are no realtors at all.
        """
        with patch.object(self.manager.fasad, 'realtors', return_value=[]):
            self.manager.synchronize()

        self.assertEqual(self.manager.count(), 0)

    def test_synchronize_does_not_delete_real_estates(self):
        """
        Should not delete real estates (by deleting realtors).
        """
        real_estate = factories.RealEstate()

        self.assertTrue(RealEstate.objects.exists())

        with patch.object(self.manager.fasad, 'realtors', return_value=[]):
            self.manager.synchronize()

        self.assertTrue(RealEstate.objects.exists())
