from django.test import TestCase
from django.utils import timezone
from django.utils.dateparse import parse_datetime, parse_date

from fasad.managers import FasAdShowingManager
from fasad.models import Showing
from fasad import factories


class FasAdShowingManagerTest(TestCase):

    @classmethod
    def setUpTestData(cls):
        cls.manager = FasAdShowingManager()
        cls.manager.model = Showing
        cls.real_estate = factories.RealEstate()

        cls.showings_zero = {"Showings": {}}

        cls.showings_single = {"Showings": {"Showing": [{
               "@startDate": "2015-08-30 14:15:00",
               "@endDate": "2015-08-30 15:00:00",
               "@comment": "Öppen visning",
        }]}}

        cls.showings_multiple = {"Showings": {"Showing": [
            {
                   "@startDate": "2015-08-30 14:15:00",
                   "@endDate": "2015-08-30 15:00:00",
                   "@comment": "Öppen visning",
            },
            {
                   "@startDate": "2015-09-30 14:15:00",
                   "@endDate": "2015-09-30 15:00:00",
                   "@comment": "Öppen visning",
            },
        ]}}

    def test_update_or_create_from_json_saves_details(self):
        showing_json = self.showings_single['Showings']['Showing'][0]

        self.manager.update_or_create_from_json(self.real_estate, self.showings_single)

        showing = self.manager.first()

        self.assertEqual(showing.start_date,
                         timezone.make_aware(parse_datetime(showing_json['@startDate'])))
        self.assertEqual(showing.end_date,
                         timezone.make_aware(parse_datetime(showing_json['@endDate'])))
        self.assertEqual(showing.comment, 'Öppen visning')

    def test_update_or_create_from_json_saves_none_start_and_end_date(self):
        json_item = {"Showings": {"Showing": [{
               "@startDate": "",
               "@endDate": "",
               "@comment": "Öppen visning",
        }]}}

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        showing = self.manager.first()

        self.assertEqual(showing.start_date, None)
        self.assertEqual(showing.end_date, None)

    def test_update_or_create_from_json_saves_dates_without_time(self):
        json_item = {"Showings": {"Showing": [{
               "@startDate": "2015-08-30",
               "@endDate": "2015-08-31",
               "@comment": "Öppen visning",
        }]}}

        self.manager.update_or_create_from_json(self.real_estate, json_item)

        showing = self.manager.first()

        self.assertEqual(showing.start_date,
                         timezone.make_aware(parse_datetime('2015-08-30 00:00:00')))
        self.assertEqual(showing.end_date,
                         timezone.make_aware(parse_datetime('2015-08-31 00:00:00')))

    def test_update_or_create_from_json_saves_multiple_showings(self):
        self.manager.update_or_create_from_json(self.real_estate, self.showings_multiple)

        self.assertEqual(self.manager.count(), 2)

    def test_update_or_create_from_json_saves_a_single_showing(self):
        self.manager.update_or_create_from_json(self.real_estate, self.showings_single)

        self.assertEqual(self.manager.count(), 1)

    def test_update_or_create_from_json_handles_zero_showings(self):
        self.manager.update_or_create_from_json(self.real_estate, self.showings_zero)

        self.assertFalse(self.manager.exists())

    def test_update_or_create_from_json_deletes_old_showings(self):
        old_showing = factories.Showing(real_estate=self.real_estate)

        self.assertTrue(self.manager.exists())

        self.manager.update_or_create_from_json(self.real_estate, self.showings_zero)

        self.assertFalse(self.manager.exists())



